
#ifndef __OBJHEADER_H
#define __OBJHEADER_H


/*----------------------------------------------------------------------
	O b j e c t   H e a d e r
----------------------------------------------------------------------*/

#define OBJHEADER \
	unsigned size  : 24; \
	unsigned flags :  8; \
	union { \
		unsigned stuff; \
		struct { \
			unsigned	locks :  8; \
			unsigned	slots : 24; \
		} count; \
		Ref destRef; \
	} gc;


typedef struct
{
	OBJHEADER
} ObjHeader;


/*----------------------------------------------------------------------
	S l o t t e d   O b j e c t
----------------------------------------------------------------------*/

typedef struct
{
	OBJHEADER

	Ref		slot[kVariableLengthArray];
} SlottedObject;

#define SIZEOF_SLOTTEDOBJECT (sizeof(SlottedObject) - sizeof(Ref)*kVariableLengthArray)


/*----------------------------------------------------------------------
	F o r w a r d i n g   O b j e c t
----------------------------------------------------------------------*/

typedef struct
{
	OBJHEADER

	Ref		obj;
} ForwardingObject;


/*----------------------------------------------------------------------
	F a u l t   O b j e c t
----------------------------------------------------------------------*/

typedef struct _TStoreWrapper TStoreWrapper;

typedef struct
{
	OBJHEADER

	long		x08;
	Ref		x0C;
	TStoreWrapper * x10;
	Ref		x14;
	Ref		x18;
} FaultObject;


/*----------------------------------------------------------------------
	B i n a r y   O b j e c t
----------------------------------------------------------------------*/

typedef struct
{
	OBJHEADER

	Ref		objClass;
	char		data[kVariableLengthArray];
} BinaryObject;

#define SIZEOF_BINARYOBJECT (sizeof(BinaryObject) - kVariableLengthArray)

#define kMaxObjSize (0x00FFFFFFL - SIZEOF_BINARYOBJECT)


/*----------------------------------------------------------------------
	L a r g e   B i n a r y   O b j e c t
----------------------------------------------------------------------*/

typedef struct LargeBinaryProcs
{
	long		(*GetLength)	(long * cookie);
	char *	(*GetData)		(long * cookie);
	void		(*SetLength)	(long * cookie, long length);
	void		(*Clone)			(long * cookie, ObjHeader * obj);
	void		(*Delete)		(long * cookie);
	void		(*SetClass)		(long * cookie);
	void		(*Mark)			(long * cookie);
	void		(*UpdateRef)	(long * cookie);
} LargeBinaryProcs;


typedef struct
{
	OBJHEADER

	Ref		objClass;
	LargeBinaryProcs * procs;
	long		data[kVariableLengthArray];
} LargeBinaryObject;

#define SIZEOF_LARGEBINARYOBJECT (sizeof(LargeBinaryObject) - sizeof(Ref)*kVariableLengthArray)


/*----------------------------------------------------------------------
	A r r a y   O b j e c t
----------------------------------------------------------------------*/

typedef struct
{
	OBJHEADER

	Ref		objClass;
	Ref		slot[kVariableLengthArray];
} ArrayObject;

#define SIZEOF_ARRAYOBJECT (sizeof(ArrayObject) - sizeof(Ref)*kVariableLengthArray)

#define kMaxSlottedObjSize (kMaxObjSize / sizeof(Ref))


/*----------------------------------------------------------------------
	F r a m e   O b j e c t
----------------------------------------------------------------------*/

typedef struct
{
	OBJHEADER

	Ref		map;
	Ref		slot[kVariableLengthArray];
} FrameObject;

#define SIZEOF_FRAMEOBJECT (sizeof(FrameObject) - sizeof(Ref)*kVariableLengthArray)


/*----------------------------------------------------------------------
	F r a m e   M a p   O b j e c t
----------------------------------------------------------------------*/

typedef struct
{
	OBJHEADER

	Ref		objClass;
	Ref		supermap;
	Ref		slot[kVariableLengthArray];
} FrameMapObject;

#define SIZEOF_FRAMEMAPOBJECT (sizeof(FrameMapObject) - sizeof(Ref)*kVariableLengthArray)


/*----------------------------------------------------------------------
	S y m b o l   O b j e c t
----------------------------------------------------------------------*/

typedef struct
{
	OBJHEADER

	Ref		objClass;
	ULong		hash;
	char		name[kVariableLengthArray];
} SymbolObject;

#define SIZEOF_SYMBOLOBJECT (sizeof(SymbolObject) - kVariableLengthArray)


/*----------------------------------------------------------------------
	S t r i n g   O b j e c t
----------------------------------------------------------------------*/

typedef struct
{
	OBJHEADER

	Ref		objClass;
	UniChar	str[kVariableLengthArray];
} StringObject;

#define SIZEOF_STRINGOBJECT (sizeof(StringObject) - sizeof(UniChar)*kVariableLengthArray)


/*----------------------------------------------------------------------
	O b j e c t   F l a g s
----------------------------------------------------------------------*/

typedef enum
{
	kObjSlotted		= 0x01,
	kObjFrame		= 0x02,
	kObjFree			= 0x04,
	kObjMarked		= 0x08,
	kObjLocked		= 0x10,
	kObjForward		= 0x20,
	kObjReadOnly	= 0x40,
	kObjDirty		= 0x80,

	kBinaryObject	= 0x00,
	kLargeBinaryObject = kObjFrame,
	kArrayObject	= kObjSlotted,
	kFrameObject	= kObjSlotted + kObjFrame,
	kObjMask			= kObjSlotted + kObjFrame
} ObjFlags;

#define kMapPlain		MAKEINT(0)
#define kMapSorted	MAKEINT(1 << 0)
#define kMapShared	MAKEINT(1 << 1)
#define kMapProto		MAKEINT(1 << 2)


/*----------------------------------------------------------------------
	R e f   < - >   P o i n t e r   C o n v e r s i o n
----------------------------------------------------------------------*/

ObjHeader *	ObjectPtr(Ref r);

#define PTR(_o) ((ObjHeader*)((Ref)_o - 1))
#define INC(_o,_i) ((ObjHeader*)((char*)_o + _i))

#define DATAPTR(_o) (ObjectPtr(_o) + sizeof(ObjHeader) / sizeof(long))
#define LENGTH(_o) (((ObjectPtr((Ref)_o))->size - sizeof(ObjHeader)) / sizeof(Ref))
#define ARRAYLENGTH(_o) ((_o->size - SIZEOF_ARRAYOBJECT) / sizeof(Ref))


#endif /* __OBJHEADER_H */
