/*
	File:			refs.h

	Contains:	Newton object system
					Basic object interface
*/

#ifndef __REFS_H
#define __REFS_H


/*----------------------------------------------------------------------
	R e f   T a g   B i t s
----------------------------------------------------------------------*/

#define kRefTagBits		  2
#define kRefValueBits	 30
#define kRefValueMask	(-1 << kRefTagBits)
#define kRefTagMask		 ~kRefValueMask

#define kRefImmedBits	 2
#define kRefImmedMask	(-1 << kRefImmedBits)

enum
{
	kTagInteger,
	kTagPointer,
	kTagImmed,
	kTagMagicPtr
};

enum
{
	kImmedSpecial,
	kImmedChar,
	kImmedBoolean,
	kImmedReserved
};


/*----------------------------------------------------------------------
	O b j e c t   T a g   F u n c t i o n s
----------------------------------------------------------------------*/

#define	MAKEINT(i)				(((long) (i)) << kRefTagBits)
#define	MAKEIMMED(t, v)		((((((long) (v)) << kRefImmedBits) | ((long) (t))) << kRefTagBits) | kTagImmed)
#define	MAKECHAR(c)				MAKEIMMED(kImmedChar, (unsigned) c)
#define	MAKEBOOLEAN(b)			(b ? TRUEREF : FALSEREF)
#define	MAKEPTR(p)				((Ref)((char*)p + 1))
#define	MAKEMAGICPTR(index)	((Ref) (((long) (index)) << kRefTagBits) | kTagMagicPtr)


/*----------------------------------------------------------------------
	I m m e d i a t e   C l a s s   C o n s t a n t s
----------------------------------------------------------------------*/

#define	kWeakArrayClass		MAKEIMMED(kImmedSpecial, 1)
#define	kFaultBlockClass		MAKEIMMED(kImmedSpecial, 2)
#define	kFuncClass				MAKEIMMED(kImmedSpecial, 3)
#define	kBadPackageRef			MAKEIMMED(kImmedSpecial, 4)
#define	kUnstreamableObject	MAKEIMMED(kImmedSpecial, 5)
#define	kSymbolClass			MAKEIMMED(kImmedSpecial, 0x5555)

#define	kPlainFuncClass		0x0032
#define	kPlainCFunctionClass	0x0132
#define	kBinCFunctionClass	0x0232

#define	ISFUNCCLASS(r)	(((r) & 0xFF) == kFuncClass)
#define	FUNCKIND(r)		(((UInt32) r) >> 8)

#define	RTAG(r)			(((Ref) (r)) & kRefTagMask)
#define	RVALUE(r)		(((Ref) (r)) >> kRefTagBits)
#define	RIMMEDTAG(r)	(RVALUE(r) & ~kRefImmedMask)
#define	RIMMEDVALUE(r)	(RVALUE(r) >> kRefImmedBits)
#define	RPTR(p)			((char*)((Ref)p & kRefValueMask))
#define	ISINT(r)			(RTAG(r) == kTagInteger)
#define	NOTINT(r)		(RTAG(r) != kTagInteger)
#define	ISPTR(r)			((BOOL)(((Ref) (r)) & 1))
#define	ISREALPTR(r)	(RTAG(r) == kTagPointer)
#define	NOTREALPTR(r)	(RTAG(r) != kTagPointer)
#define	ISMAGICPTR(r)	(RTAG(r) == kTagMagicPtr)
#define	ISIMMED(r)		(RTAG(r) == kTagImmed)
#define	ISCHAR(r)		(ISIMMED(r) && (RIMMEDTAG(r) == kImmedChar))
#define	ISBOOLEAN(r)	(ISIMMED(r) && (RIMMEDTAG(r) == kImmedBoolean))
	

#endif // __REFS_H
