/*------------------------------------------------------------------------------
	Newton interface to some useful utility functions.
------------------------------------------------------------------------------*/

#include "Objects.h"
#include "Unicode.h"
#include "NewtonDebug.h"

extern	unsigned char	ToHex(UniChar ch);


/*----------------------------------------------------------------------
	D a t a
----------------------------------------------------------------------*/

static	const char	hexChars[16] = { '0','1','2','3','4','5','6','7',
												  '8','9','A','B','C','D','E','F' };


/*----------------------------------------------------------------------
	Munge an array of bytes into a binary object.

	Args:		ioData
					binary object
				inOffset
					integer offset at which to place bytes
				inBytes
					array of bytes

	Return:	modified ioData
----------------------------------------------------------------------*/

extern "C"
Ref MungeBytes(RefArg rcvr, RefArg ioData, RefArg inOffset, RefArg inBytes)
{
	if (!IsBinary(ioData))
		ThrowBadTypeWithFrameData(kNSErrNotABinaryObject, ioData);
	if (!IsInt(inOffset))
		ThrowBadTypeWithFrameData(kNSErrNotAnInteger, inOffset);
	if (!IsArray(inBytes))
		ThrowBadTypeWithFrameData(kNSErrNotAnArray, inBytes);

	long byteLen = Length(inBytes);
	long offset = RefToInt(inOffset);
	if (byteLen > Length(ioData) - offset)
		ThrowBadTypeWithFrameData(kNSErrOutOfRange, MAKEINT(byteLen));

//printf("munging %d bytes:\n", byteLen);
	WITH_LOCKED_BINARY(ioData, dataPtr)
		WITH_LOCKED_BINARY(inBytes, bytePtr)
			const Ref * srcPtr = (const Ref *) bytePtr;
			UChar * dstPtr = (UChar *) dataPtr + offset;
			while (byteLen-- > 0)
			{
				*dstPtr++ = RefToInt(*srcPtr++);
//printf("%d\n", *(dstPtr - 1));
			}
		END_WITH_LOCKED_BINARY(inBytes)
	END_WITH_LOCKED_BINARY(ioData)
//printf("munging complete\n");

	return ioData;
}


/*----------------------------------------------------------------------
	Convert a Mac OSType signature from the usual 4 characters
	to hex representation.

	Args:		inStr
					string containing char representation of OSType

	Return:	string containing hex representation of OSType
----------------------------------------------------------------------*/

extern "C"
Ref HexOSTypeString(RefArg rcvr, RefArg inStr)
{
	RefVar charStr(inStr);
	if (!IsString(charStr))
		charStr = MakeString("????");

	char typeStr[9];
	strcpy(typeStr, "20202020");
	WITH_LOCKED_BINARY(charStr, charPtr)
		UniChar * srcPtr = (UniChar *) charPtr;
		char * dstPtr = typeStr;
		UniChar ch;
		for (int i = 0; i < 4; i++)
		{
			ch = *srcPtr++;
			if (ch == 0)
				break;
			*dstPtr++ = hexChars[(ch >> 4) & 0x0F];
			*dstPtr++ = hexChars[ch & 0x0F];
		}
	END_WITH_LOCKED_BINARY(charStr)

	return MakeString(typeStr);
}


/*----------------------------------------------------------------------
	Convert a Mac OSType signature from hex representation
	to 4 characters.

	Args:		inHex
					string containing hex representation of OSType

	Return:	string containing char representation of OSType
----------------------------------------------------------------------

extern "C"
Ref CharOSTypeString(RefArg rcvr, RefArg inStr)
{
	RefVar typeStr(MakeString("????"));

	if (IsString(inStr))
		WITH_LOCKED_BINARY(inStr, inStrPtr)
			WITH_LOCKED_BINARY(typeStr, typePtr)
				const UniChar * srcPtr = (UniChar *) inStrPtr;
				UniChar * dstPtr = (UniChar *) typePtr;
				UniChar ch1, ch2;
				for (int i = 0; i < 4; i++)
				{
					ch1 = *srcPtr++;
					if (ch1 == 0)
						break;
					ch2 = *srcPtr++;
					if (ch2 == 0)
						break;
					*dstPtr++ = (ToHex(ch1) << 4) | ToHex(ch2);
				}
			END_WITH_LOCKED_BINARY(typeStr)
		END_WITH_LOCKED_BINARY(inStr)

	return typeStr;
}
*/